#include <fstream>
#include <iostream>
#include <cstring>
#include <list>
#include <string>
#include <stack>
#include <algorithm>
using namespace std;

const int MaxSize = 5*(2<<20)+10000;

typedef unsigned char byte;

template<class T>
inline T& Min(T v1, T v2) {return (v1 < v2) ? v1 : v2;}

#ifdef _MSC_VER
#pragma warning (disable: 4786)
#endif


const byte Unknown  = 255;

typedef byte Color;
const Color Red     = 0;
const Color Green   = 1;
const Color Blue    = 2;
const Color Cyan    = 3;
const Color Magenta = 4;
const Color Yellow  = 5;
const Color Black   = 6;
const Color White   = 7;

typedef byte Flag;
const Flag Bold   = 0x01;
const Flag Empha  = 0x02;
const Flag Italic = 0x04;
const Flag Strong = 0x08;
const Flag Type   = 0x10;
const Flag Plain  = 0x20;

class State
{
public:
	State():_size(Unknown), _color(Unknown), _under(0), _flags(0) {}

	void AddState(char c);
	void GetOpenString (char* str); // sizeof str >= 27
	void GetCloseString(char* str); // sizeof str >= 27

	bool SpaceMatch(const State& state) const;
	int GetSameCount(const State& state) const;
	int GetSameCount(const State& state1, const State& state2) const;
	int GetPLEffectedTagCo() const;
	void SuckCommon(State& st1, State& st2, bool suckColor = true, bool suckSize = true);

	byte GetUnder() const {return _under;}
	void SetUnder(byte under) {_under = under;}
	void IncUnder() {++_under;}
	void DecUnder() {--_under;}

	byte GetSize() const {return _size;}
	void SetSize(byte size) {_size = size;}

	Color GetColor() const {return _color;}
	void SetColor(Color color) {_color = color;}

	Flag GetFlags() const {return _flags;}
	void ResetFlags(Flag flags = 0) {_flags = flags;}

	Flag IsBold  () const {return _flags & Bold    ;}
	Flag IsEmpha () const {return _flags & Empha   ;}
	Flag IsItalic() const {return _flags & Italic  ;}
	Flag IsStrong() const {return _flags & Strong  ;}
	Flag IsType  () const {return _flags & Type    ;}
	Flag IsPlain () const {return _flags & Plain   ;}
	Flag IsUnder () const {return _under != 0      ;}
	Flag IsSize  () const {return _size  != Unknown && _size != Unknown;}
	Flag IsColor () const {return _color != Unknown && _color != Unknown;}

	void SetBold  () {_flags |= Bold  ;}
	void SetItalic() {_flags |= Italic;}
	void SetType  () {_flags |= Type  ;}
	void SetEmpha () {if (!IsStrong()) _flags |= Empha ;}
	void SetStrong() {_flags |= Strong; _flags &= ~Empha;}
	void SetPlain () {_flags |= Plain;}

	void SetBold  (bool b) {_flags = (_flags & ~Bold  ) | (b << 0);}
	void SetItalic(bool b) {_flags = (_flags & ~Italic) | (b << 2);}
	void SetType  (bool b) {_flags = (_flags & ~Type  ) | (b << 4);}
	void SetEmpha (bool b) {if (!IsStrong()) _flags = (_flags & ~Empha ) | (b << 1);}
	void SetStrong(bool b) {if (b) SetStrong(); else _flags &= ~Strong;}

	void Reset() {_size = _color = Unknown, _under = _flags = 0;}

	bool operator==(const State& st) const
		{return GetSize() == st.GetSize() && GetUnder() == st.GetUnder() &&
			GetFlags() == st.GetFlags() && GetColor() == st.GetColor();}
	bool operator!=(const State& st) const {return !(*this == st);}

protected:
	void AddOpen(char c, char*& str) {*str = '<'; *++str = c, *++str = '>'; ++str;}
	void AddOpen(char c1, char c2, char*& str) {*str = '<'; *++str = c1, *++str = c2, *++str = '>'; ++str;}
	void AddClose(char c, char*& str) {*str = '<'; *++str = '/'; *++str = c, *++str = '>'; ++str;}
	void AddClose(char c1, char c2, char*& str) {*str = '<'; *++str = '/'; *++str = c1, *++str = c2, *++str = '>'; ++str;}

	byte  _size;
	byte  _under;
	Flag  _flags;
	Color _color;
};

class Node
{
public:
	Node(State prior = State(), State change = State(), const char* text = 0):
		_prior(prior), _change(change), _text(text) {}
	~Node() {}

	typedef list<Node*> Cont;
	typedef Cont::iterator Iter;
	typedef Cont::const_iterator CIter;

	State& GetChange() {return _change;}
	const State& GetChange() const {return _change;}

	State& GetPrior() {return _prior;}
	const State& GetPrior() const {return _prior;}

	Cont& GetChildren() {return _children;}
	const Cont& GetChildren() const {return _children;}

	const char* GetText() const {return _text;}

	void AddChild(Node* node) {_children.push_back(node);}

protected:
	const char* _text;
	State _prior, _change;
	list<Node*> _children;
};


class Parser
{
public:
	Parser() {}
	~Parser() {};

	Node* Parse(istream& in, char* back);

protected:
	void AddNode(Node::Cont& nodes, const State& state, char* beg, char* end);
	static bool PureSpace(string& str);

	string _lastText;
	State _lastState;
	bool _lastPureSpace;
	bool _lastSpace;
};

Node* Parser::Parse(istream& in, char* back)
{
	Node* parent = new Node();
	Node::Cont& nodes = parent->GetChildren();

	char* buffer = new char[MaxSize];
	char* end = buffer;
	streambuf* buf = in.rdbuf();

	stack<State> states;
	State state;

	bool doit = false;
	bool lastWhite = false;
	int c;
	for (;;)
	{
		c = buf->sbumpc();
		if (c == EOF)
			break;
		*back++ = c;

		if (c == '<')
		{
			c = buf->sbumpc();
			*back++ = c;
			if (c == '/')
			{
				AddNode(nodes, state, buffer, end);
				end = buffer;
				state = states.top();
				states.pop();
			}
			else
			{
				AddNode(nodes, state, buffer, end);
				end = buffer;
				states.push(state);
				state.AddState(c);
			}

			do
			{
				c = buf->sbumpc();
				*back++ = c;
			}
			while (c != '>');

			continue;
		}
		if (lastWhite && !state.IsType() && isspace(c))
			continue;
		else
			lastWhite = (isspace(c) && !state.IsType());

		*end = c;
		++end;
	}

	AddNode(nodes, state, buffer, end);

	if (!_lastText.empty())
	{
		char* str = new char[_lastText.length() + 1];
		strcpy(str, _lastText.c_str());

		nodes.push_back(new Node(_lastState, _lastState, str));
	}
	*back = '\0';
	return parent;
}

bool Parser::PureSpace(string& str)
{
	string::const_pointer it  = str.begin();
	string::const_pointer end = str.end();
	for (; it != end; ++it)
		if (!isspace(*it))
			return false;
	return true;
}

void Parser::AddNode(Node::Cont& nodes, const State& state, char* beg, char* end)
{
	if (beg == end)
		return;

	string text(beg, end);
	bool pureSpace = PureSpace(text);
	bool lastSpace = (isspace(*text.rend()) != 0);

	if (!_lastText.empty() && _lastState.SpaceMatch(state))
	{
		if (state.IsType())
		{
			if (_lastPureSpace)
			{
				_lastPureSpace = pureSpace;
				_lastSpace = lastSpace;
				_lastState = state;
				_lastText += text;
				return;
			}
			else if (pureSpace)
			{
				_lastSpace = true;
				_lastText += text;
				return;
			}
		}
		else
		{
			if (_lastPureSpace && !isspace(text[0]))
			{
				_lastPureSpace = pureSpace;
				_lastSpace = lastSpace;
				_lastState = state;
				_lastText += text;
				return;
			}
			else if (pureSpace && !_lastSpace)
			{
				_lastSpace = true;
				_lastText += text;
				return;
			}
		}
	}

	if (!_lastText.empty())
	{
		char* str = new char[_lastText.length() + 1];
		strcpy(str, _lastText.c_str());

		nodes.push_back(new Node(_lastState, _lastState, str));
	}
	_lastState = state;
	_lastPureSpace = pureSpace;
	_lastSpace = lastSpace;
	_lastText.swap(text);
}

void State::AddState(char c)
{
	switch (c)
	{
	case 'B': SetBold()  ; break;
	case 'I': SetItalic(); break;
	case 'T': SetType()  ; break;
	case 'S': SetStrong(); break;
	case 'E': SetEmpha(!IsEmpha()) ; break;
	case 'U': IncUnder() ; break;
	case 'P': _under = 0, _flags = 0; break;

	case 'r': SetColor(Red)    ; break;
	case 'g': SetColor(Green)  ; break;
	case 'b': SetColor(Blue)   ; break;
	case 'c': SetColor(Cyan)   ; break;
	case 'm': SetColor(Magenta); break;
	case 'y': SetColor(Yellow) ; break;
	case 'k': SetColor(Black)  ; break;
	case 'w': SetColor(White)  ; break;

	default: // [0..9]
		SetSize(c - '0');
		break;
	}
}

void State::GetOpenString(char* str)
{
	if (IsPlain ()) AddOpen('P', 'L', str);
	if (IsBold  ()) AddOpen('B',      str);
	if (IsEmpha ()) AddOpen('E', 'M', str);
	if (IsItalic()) AddOpen('I',      str);
	if (IsStrong()) AddOpen('S',      str);
	if (IsType  ()) AddOpen('T', 'T', str);

	for (int i = 0; i < _under; ++i)
		AddOpen('U', str);

	if (IsColor())
	{
		switch (_color)
		{
		case Red    : AddOpen('r', str); break;
		case Green  : AddOpen('g', str); break;
		case Blue   : AddOpen('b', str); break;
		case Cyan   : AddOpen('c', str); break;
		case Magenta: AddOpen('m', str); break;
		case Yellow : AddOpen('y', str); break;
		case Black  : AddOpen('k', str); break;
		case White  : AddOpen('w', str); break;
		}
	}

	if (IsSize())
		AddOpen(_size + '0', str);
	*str = '\0';
}

void State::GetCloseString(char* str)
{
	if (IsSize())
		AddClose(_size + '0', str);

	if (IsColor())
	{
		switch (_color)
		{
		case Red    : AddClose('r', str); break;
		case Green  : AddClose('g', str); break;
		case Blue   : AddClose('b', str); break;
		case Cyan   : AddClose('c', str); break;
		case Magenta: AddClose('m', str); break;
		case Yellow : AddClose('y', str); break;
		case Black  : AddClose('k', str); break;
		case White  : AddClose('w', str); break;
		}
	}

	for (int i = 0; i < _under; ++i)
		AddClose('U', str);

	if (IsType  ()) AddClose('T', 'T', str);
	if (IsStrong()) AddClose('S',      str);
	if (IsItalic()) AddClose('I',      str);
	if (IsEmpha ()) AddClose('E', 'M', str);
	if (IsBold  ()) AddClose('B',      str);
	if (IsPlain ()) AddClose('P', 'L', str);
	*str = '\0';
}

bool State::SpaceMatch(const State& state) const
{
	return
		IsType()  == state.IsType()  &&
		GetSize()  == state.GetSize()  &&
		GetUnder() == state.GetUnder() &&
		(GetUnder() == 0 || GetColor() == state.GetColor());
}

int State::GetSameCount(const State& st) const
{
	Flag flags = GetFlags() & st.GetFlags();

	return
		(IsColor() && GetColor() == st.GetColor()) +
		(IsSize()  && GetSize () == st.GetSize ()) +

		Min(GetUnder(), st.GetUnder()) +

		((flags & Type  ) != 0) +
		((flags & Strong) != 0) +
		((flags & Italic) != 0) +
		((flags & Empha ) != 0) + 
		((flags & Bold  ) != 0) +
		((flags & Plain ) != 0);
}


int State::GetSameCount(const State& st1, const State& st2) const
{
	Flag flags = GetFlags() & st1.GetFlags() & st2.GetFlags();

	return
		(IsColor() && GetColor() == st1.GetColor() && GetColor() == st2.GetColor()) +
		(IsSize()  && GetSize () == st1.GetSize () && GetColor() == st2.GetColor()) +

		Min(GetUnder(), Min(st1.GetUnder(), st2.GetUnder())) +

		((flags & Type  ) != 0) +
		((flags & Strong) != 0) +
		((flags & Italic) != 0) +
		((flags & Empha ) != 0) + 
		((flags & Bold  ) != 0) +
		((flags & Plain ) != 0);
}

int State::GetPLEffectedTagCo() const
{
	return GetUnder()    +
		(IsType  () != 0) +
		(IsStrong() != 0) +
		(IsItalic() != 0) +
		(IsEmpha () != 0) +
		(IsBold  () != 0);
}

void State::SuckCommon(State& st1, State& st2, bool suckColor, bool suckSize)
{
	ResetFlags(st1.GetFlags() & st2.GetFlags());
	st1.ResetFlags(st1.GetFlags() & ~GetFlags());
	st2.ResetFlags(st2.GetFlags() & ~GetFlags());
	
	SetUnder(st1.GetUnder() <  st2.GetUnder() ? st1.GetUnder() : st2.GetUnder());
	st1.SetUnder(st1.GetUnder() -  GetUnder());
	st2.SetUnder(st2.GetUnder() -  GetUnder());

	if (suckColor)
	{
		SetColor(st1.GetColor() == st2.GetColor() ? st1.GetColor() : Unknown);
		if (st1.GetColor() == GetColor()) st1.SetColor(Unknown);
		if (st2.GetColor() == GetColor()) st2.SetColor(Unknown);
	}

	if (suckSize)
	{
		SetSize (st1.GetSize () == st2.GetSize () ? st1.GetSize () : Unknown);
		if (st1.GetSize () == GetSize ()) st1.SetSize (Unknown);
		if (st2.GetSize () == GetSize ()) st2.SetSize (Unknown);
	}
}


struct BufStream
{
	BufStream() {_end = _buf = new char[MaxSize];}
	~BufStream() {delete[] _buf;}

	BufStream& operator<<(const char* str)
	{
		for (; *str != '\0'; ++str, ++_end)
			*_end = *str;
		return *this;
	}

	void Term()
	{
		*_end = '\0';
	}

	char* _buf;
	char* _end;
};

void PrintSML(Node* node, BufStream& out)
{
	static char tmp[30];
	node->GetChange().GetOpenString(tmp);
	out << tmp;

	if (node->GetText() != 0)
		out << node->GetText();

	Node::Iter it  = node->GetChildren().begin();
	Node::Iter end = node->GetChildren().end();
	for (; it != end; ++it)
		PrintSML(*it, out);

	node->GetChange().GetCloseString(tmp);
	out << tmp;
}

void MergeIdentical(Node* node)
{
	Node::Cont& nodes = node->GetChildren();
	Node::Iter it1 = nodes.begin();
	Node::Iter end = nodes.end();

	if (it1 == end)
		return;

	Node::Iter it2 = it1;
	++it2;

	while (it2 != end)
	{
		if ((*it1)->GetChange() == (*it2)->GetChange())
		{
			(*it2)->GetChange().Reset();
			(*it1)->AddChild(*it2);
			it2 = nodes.erase(it2);
		}
		else
			++it1, ++it2;
	}
}

inline void MergeAdj(Node::Cont& nodes, Node::Iter& it1, Node::Iter& it2, bool suckColor = true, bool suckSize = true)
{
	State ch;
	ch.SuckCommon((*it1)->GetChange(), (*it2)->GetChange(), suckColor, suckSize);
	Node* node = new Node(ch, ch);

	node->AddChild(*it1);
	node->AddChild(*it2);
	nodes.insert(it1, node);

	it1 = nodes.erase(it1);
	--it1;
	it2 = nodes.erase(it2);
}

inline int SameCount(Node::Iter& it1, Node::Iter& it2)
{
	return (*it1)->GetChange().GetSameCount((*it2)->GetChange());
}

inline int SameCount(Node::Iter& it1, Node::Iter& it2, Node::Iter& it3)
{
	return (*it1)->GetChange().GetSameCount((*it2)->GetChange(), (*it3)->GetChange());
}

bool MergeSimilar(Node* node, int minSimCo = 1)
{
	Node::Cont& nodes = node->GetChildren();
	Node::Iter it1 = nodes.begin();
	Node::Iter end = nodes.end();

	if (it1 == end)
		return false;

	Node::Iter it2 = it1;
	++it2;

	bool doneSome = false;
	while (it2 != end)
	{
		if (SameCount(it1, it2) >= minSimCo)
		{
			MergeAdj(nodes, it1, it2);
			doneSome = true;
		}
		else
			++it1, ++it2;
	}

	return doneSome;
}

bool DoPL(Node* node, int minGainCo = 1)
{
	Node::Cont& nodes = node->GetChildren();
	Node::Iter it1 = nodes.begin();
	Node::Iter end = nodes.end();

	if (it1 == end)
		return false;

	Node::Iter it2 = it1;
	++it2;
	if (it2 == end)
		return false;

	Node::Iter it3 = it2;
	++it3;

	bool doneSome = false;
	while (it3 != end)
	{
		int gain = SameCount(it1, it3) - SameCount(it1, it2, it3) - (*it2)->GetChange().GetPLEffectedTagCo() - 1;

		if (gain >= minGainCo)
		{
			if ((*it1)->GetChange().GetColor() == (*it3)->GetChange().GetColor() &&
				(*it1)->GetChange().GetColor() != Unknown && (*it2)->GetChange().GetColor() == Unknown)
					--gain;

			if ((*it1)->GetChange().GetSize() == (*it3)->GetChange().GetSize() &&
				(*it1)->GetChange().GetSize() != Unknown && (*it2)->GetChange().GetSize() == Unknown)
					--gain;
		}

		if (gain >= minGainCo)
		{
			State ch;
			ch.SuckCommon((*it1)->GetChange(), (*it3)->GetChange(),
				(*it2)->GetChange().GetColor() != Unknown, (*it2)->GetChange().GetSize() == Unknown);
			Node* node = new Node(ch, ch);
			(*it2)->GetChange().SetPlain();

			node->AddChild(*it1);
			node->AddChild(*it2);
			node->AddChild(*it3);
			nodes.insert(it1, node);

			it3 = nodes.erase(it3);
			it2 = nodes.erase(it2);
			it1 = nodes.erase(it1);
			++it2;
			if (it2 == end)
				break;
			it3 = it2;
			++it3;

			doneSome = true;
		}
		else
			++it1, ++it2, ++it3;
	}
	return doneSome;
}

void DoSize(Node::Cont& nodes)
{
	Node::Iter it = nodes.begin();
	Node::Iter end = nodes.end();

	Node::Iter last[10], back[10][10];
	fill_n(last, 10, end);

	for (; it != end; ++it)
	{
		State st = (*it)->GetPrior();
		if (st.GetSize() == Unknown)
			fill_n(last, 10, end);
		else if (last[st.GetSize()] == end)
		{
			copy(last, last + 10, back[st.GetSize()]);
			last[st.GetSize()] = it;
		}
		else
		{
			(*last[st.GetSize()])->GetChange().SetSize(Unknown);
			(*it)->GetChange().SetSize(Unknown);

			State nst;
			nst.SetSize(st.GetSize());

			Node* node  = new Node(nst, nst);
			Node::Iter tmp = nodes.insert(last[st.GetSize()], node);
			node->GetChildren().splice(node->GetChildren().end(), nodes,
				last[st.GetSize()], ++it);
			--it;

			copy(back[st.GetSize()], back[st.GetSize()] + 10, last);
			last[st.GetSize()] = tmp;
		}
	}
}

void DoColor(Node::Cont& nodes)
{
	Node::Iter it = nodes.begin();
	Node::Iter end = nodes.end();

	Node::Iter last[10], back[10][10];
	fill_n(last, 10, end);

	for (; it != end; ++it)
	{
		State st = (*it)->GetPrior();
		if (st.GetColor() == Unknown)
			fill_n(last, 10, end);
		else if (last[st.GetColor()] == end)
		{
			copy(last, last + 10, back[st.GetColor()]);
			last[st.GetColor()] = it;
		}
		else
		{
			(*last[st.GetColor()])->GetChange().SetColor(Unknown);
			(*it)->GetChange().SetColor(Unknown);

			State nst;
			nst.SetColor(st.GetColor());

			Node* node  = new Node(nst, nst);
			Node::Iter tmp = nodes.insert(last[st.GetColor()], node);
			node->GetChildren().splice(node->GetChildren().end(), nodes,
				last[st.GetColor()], ++it);
			--it;

			copy(back[st.GetColor()], back[st.GetColor()] + 10, last);
			last[st.GetColor()] = tmp;
		}
	}
}

int main()
{
	Node* parent;

	char* oldSML = new char[MaxSize];

	Parser parser;
	parent = parser.Parse(cin, oldSML);

	MergeIdentical(parent);
	for (int co = 10; co >= 1; --co)
	{
		do
		{
			while (MergeSimilar(parent, co))
				;
		} while (DoPL(parent, co));
	}
	DoColor(parent->GetChildren());
	DoSize(parent->GetChildren());

	BufStream bufOut;
	PrintSML(parent, bufOut);
	bufOut.Term();
	char* newSML = bufOut._buf;

	int oldSize = strlen(oldSML), newSize = strlen(newSML);
	streambuf* fbuf = cout.rdbuf();
	if (newSize < oldSize)
		fbuf->sputn(newSML, newSize);
	else
		fbuf->sputn(oldSML, oldSize);

	return 0;
}
